# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: leader.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='leader.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cleader.proto\x1a\x1cgoogle/protobuf/struct.proto\"V\n\x15WorkerRegisterRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x16\n\x0eserver_address\x18\x03 \x01(\t\"l\n\x16WorkerRegisterResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12+\n\njob_config\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\"<\n\x13WorkerUpdateRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\x03\"i\n\x14WorkerUpdateResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12*\n\tcompleted\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\"Y\n\x17WorkerDeregisterRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x17\n\x0freporter_fields\x18\x03 \x03(\t\"A\n\x18WorkerDeregisterResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\">\n\x15LeaderRegisterRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\"?\n\x16LeaderRegisterResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\":\n\x11PollLeaderRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\"N\n\x12PollLeaderResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x11\n\tcompleted\x18\x03 \x01(\x08\"d\n\x14PollLeaderJobRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\x12\x15\n\rcurrent_stage\x18\x04 \x01(\t\"Q\n\x15PollLeaderJobResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x11\n\tcompleted\x18\x03 \x01(\x08\"f\n\x14PipelineStageRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\x12\x17\n\x0fstage_completed\x18\x04 \x01(\t\">\n\x15PipelineStageResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\"~\n\x13NewJobLeaderRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12+\n\njob_config\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x13\n\x0bjob_timeout\x18\x04 \x01(\x03\"M\n\x14NewJobLeaderResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\"O\n\x16LeaderHeartbeatRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"P\n\x17LeaderHeartbeatResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xa8\x01\n\x10LeaderJobRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\x12+\n\njob_config\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x30\n\x0fpipeline_status\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"\xa9\x01\n\x11LeaderJobResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\x12+\n\njob_config\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x30\n\x0fpipeline_status\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct2\xa5\x05\n\x11\x44istributedServer\x12>\n\tAddClient\x12\x16.WorkerRegisterRequest\x1a\x17.WorkerRegisterResponse\"\x00\x12:\n\tGetUpdate\x12\x14.WorkerUpdateRequest\x1a\x15.WorkerUpdateResponse\"\x00\x12\x45\n\x0cRemoveClient\x12\x18.WorkerDeregisterRequest\x1a\x19.WorkerDeregisterResponse\"\x00\x12\x43\n\x0eRegisterLeader\x12\x16.LeaderRegisterRequest\x1a\x17.LeaderRegisterResponse\"\x00\x12\x37\n\nPollLeader\x12\x12.PollLeaderRequest\x1a\x13.PollLeaderResponse\"\x00\x12\x45\n\x12MarkStageCompleted\x12\x15.PipelineStageRequest\x1a\x16.PipelineStageResponse\"\x00\x12@\n\rPollLeaderJob\x12\x15.PollLeaderJobRequest\x1a\x16.PollLeaderJobResponse\"\x00\x12=\n\x0c\x43reateNewJob\x12\x14.NewJobLeaderRequest\x1a\x15.NewJobLeaderResponse\"\x00\x12K\n\x14\x43heckLeaderHeartbeat\x12\x17.LeaderHeartbeatRequest\x1a\x18.LeaderHeartbeatResponse\"\x00\x12:\n\x0fShareCurrentJob\x12\x11.LeaderJobRequest\x1a\x12.LeaderJobResponse\"\x00\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])




_WORKERREGISTERREQUEST = _descriptor.Descriptor(
  name='WorkerRegisterRequest',
  full_name='WorkerRegisterRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='WorkerRegisterRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='WorkerRegisterRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='server_address', full_name='WorkerRegisterRequest.server_address', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=132,
)


_WORKERREGISTERRESPONSE = _descriptor.Descriptor(
  name='WorkerRegisterResponse',
  full_name='WorkerRegisterResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='WorkerRegisterResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='WorkerRegisterResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_config', full_name='WorkerRegisterResponse.job_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=242,
)


_WORKERUPDATEREQUEST = _descriptor.Descriptor(
  name='WorkerUpdateRequest',
  full_name='WorkerUpdateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='WorkerUpdateRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='WorkerUpdateRequest.host_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=244,
  serialized_end=304,
)


_WORKERUPDATERESPONSE = _descriptor.Descriptor(
  name='WorkerUpdateResponse',
  full_name='WorkerUpdateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='WorkerUpdateResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='WorkerUpdateResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='completed', full_name='WorkerUpdateResponse.completed', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=306,
  serialized_end=411,
)


_WORKERDEREGISTERREQUEST = _descriptor.Descriptor(
  name='WorkerDeregisterRequest',
  full_name='WorkerDeregisterRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='WorkerDeregisterRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='WorkerDeregisterRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reporter_fields', full_name='WorkerDeregisterRequest.reporter_fields', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=413,
  serialized_end=502,
)


_WORKERDEREGISTERRESPONSE = _descriptor.Descriptor(
  name='WorkerDeregisterResponse',
  full_name='WorkerDeregisterResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='WorkerDeregisterResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='WorkerDeregisterResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=504,
  serialized_end=569,
)


_LEADERREGISTERREQUEST = _descriptor.Descriptor(
  name='LeaderRegisterRequest',
  full_name='LeaderRegisterRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='LeaderRegisterRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='LeaderRegisterRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=571,
  serialized_end=633,
)


_LEADERREGISTERRESPONSE = _descriptor.Descriptor(
  name='LeaderRegisterResponse',
  full_name='LeaderRegisterResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='LeaderRegisterResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='LeaderRegisterResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=635,
  serialized_end=698,
)


_POLLLEADERREQUEST = _descriptor.Descriptor(
  name='PollLeaderRequest',
  full_name='PollLeaderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='PollLeaderRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='PollLeaderRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=700,
  serialized_end=758,
)


_POLLLEADERRESPONSE = _descriptor.Descriptor(
  name='PollLeaderResponse',
  full_name='PollLeaderResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='PollLeaderResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='PollLeaderResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='completed', full_name='PollLeaderResponse.completed', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=760,
  serialized_end=838,
)


_POLLLEADERJOBREQUEST = _descriptor.Descriptor(
  name='PollLeaderJobRequest',
  full_name='PollLeaderJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='PollLeaderJobRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='PollLeaderJobRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_id', full_name='PollLeaderJobRequest.job_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='current_stage', full_name='PollLeaderJobRequest.current_stage', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=840,
  serialized_end=940,
)


_POLLLEADERJOBRESPONSE = _descriptor.Descriptor(
  name='PollLeaderJobResponse',
  full_name='PollLeaderJobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='PollLeaderJobResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='PollLeaderJobResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='completed', full_name='PollLeaderJobResponse.completed', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=942,
  serialized_end=1023,
)


_PIPELINESTAGEREQUEST = _descriptor.Descriptor(
  name='PipelineStageRequest',
  full_name='PipelineStageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='PipelineStageRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='PipelineStageRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_id', full_name='PipelineStageRequest.job_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stage_completed', full_name='PipelineStageRequest.stage_completed', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1025,
  serialized_end=1127,
)


_PIPELINESTAGERESPONSE = _descriptor.Descriptor(
  name='PipelineStageResponse',
  full_name='PipelineStageResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='PipelineStageResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='PipelineStageResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1129,
  serialized_end=1191,
)


_NEWJOBLEADERREQUEST = _descriptor.Descriptor(
  name='NewJobLeaderRequest',
  full_name='NewJobLeaderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='NewJobLeaderRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='NewJobLeaderRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_config', full_name='NewJobLeaderRequest.job_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_timeout', full_name='NewJobLeaderRequest.job_timeout', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1193,
  serialized_end=1319,
)


_NEWJOBLEADERRESPONSE = _descriptor.Descriptor(
  name='NewJobLeaderResponse',
  full_name='NewJobLeaderResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='NewJobLeaderResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='NewJobLeaderResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_id', full_name='NewJobLeaderResponse.job_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1321,
  serialized_end=1398,
)


_LEADERHEARTBEATREQUEST = _descriptor.Descriptor(
  name='LeaderHeartbeatRequest',
  full_name='LeaderHeartbeatRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='LeaderHeartbeatRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='LeaderHeartbeatRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='LeaderHeartbeatRequest.status', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1400,
  serialized_end=1479,
)


_LEADERHEARTBEATRESPONSE = _descriptor.Descriptor(
  name='LeaderHeartbeatResponse',
  full_name='LeaderHeartbeatResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='LeaderHeartbeatResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='LeaderHeartbeatResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='LeaderHeartbeatResponse.status', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1481,
  serialized_end=1561,
)


_LEADERJOBREQUEST = _descriptor.Descriptor(
  name='LeaderJobRequest',
  full_name='LeaderJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='LeaderJobRequest.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='LeaderJobRequest.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_id', full_name='LeaderJobRequest.job_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_config', full_name='LeaderJobRequest.job_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pipeline_status', full_name='LeaderJobRequest.pipeline_status', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1564,
  serialized_end=1732,
)


_LEADERJOBRESPONSE = _descriptor.Descriptor(
  name='LeaderJobResponse',
  full_name='LeaderJobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_address', full_name='LeaderJobResponse.host_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host_id', full_name='LeaderJobResponse.host_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_id', full_name='LeaderJobResponse.job_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_config', full_name='LeaderJobResponse.job_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pipeline_status', full_name='LeaderJobResponse.pipeline_status', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1735,
  serialized_end=1904,
)

_WORKERREGISTERRESPONSE.fields_by_name['job_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_WORKERUPDATERESPONSE.fields_by_name['completed'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_NEWJOBLEADERREQUEST.fields_by_name['job_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_LEADERJOBREQUEST.fields_by_name['job_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_LEADERJOBREQUEST.fields_by_name['pipeline_status'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_LEADERJOBRESPONSE.fields_by_name['job_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_LEADERJOBRESPONSE.fields_by_name['pipeline_status'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
DESCRIPTOR.message_types_by_name['WorkerRegisterRequest'] = _WORKERREGISTERREQUEST
DESCRIPTOR.message_types_by_name['WorkerRegisterResponse'] = _WORKERREGISTERRESPONSE
DESCRIPTOR.message_types_by_name['WorkerUpdateRequest'] = _WORKERUPDATEREQUEST
DESCRIPTOR.message_types_by_name['WorkerUpdateResponse'] = _WORKERUPDATERESPONSE
DESCRIPTOR.message_types_by_name['WorkerDeregisterRequest'] = _WORKERDEREGISTERREQUEST
DESCRIPTOR.message_types_by_name['WorkerDeregisterResponse'] = _WORKERDEREGISTERRESPONSE
DESCRIPTOR.message_types_by_name['LeaderRegisterRequest'] = _LEADERREGISTERREQUEST
DESCRIPTOR.message_types_by_name['LeaderRegisterResponse'] = _LEADERREGISTERRESPONSE
DESCRIPTOR.message_types_by_name['PollLeaderRequest'] = _POLLLEADERREQUEST
DESCRIPTOR.message_types_by_name['PollLeaderResponse'] = _POLLLEADERRESPONSE
DESCRIPTOR.message_types_by_name['PollLeaderJobRequest'] = _POLLLEADERJOBREQUEST
DESCRIPTOR.message_types_by_name['PollLeaderJobResponse'] = _POLLLEADERJOBRESPONSE
DESCRIPTOR.message_types_by_name['PipelineStageRequest'] = _PIPELINESTAGEREQUEST
DESCRIPTOR.message_types_by_name['PipelineStageResponse'] = _PIPELINESTAGERESPONSE
DESCRIPTOR.message_types_by_name['NewJobLeaderRequest'] = _NEWJOBLEADERREQUEST
DESCRIPTOR.message_types_by_name['NewJobLeaderResponse'] = _NEWJOBLEADERRESPONSE
DESCRIPTOR.message_types_by_name['LeaderHeartbeatRequest'] = _LEADERHEARTBEATREQUEST
DESCRIPTOR.message_types_by_name['LeaderHeartbeatResponse'] = _LEADERHEARTBEATRESPONSE
DESCRIPTOR.message_types_by_name['LeaderJobRequest'] = _LEADERJOBREQUEST
DESCRIPTOR.message_types_by_name['LeaderJobResponse'] = _LEADERJOBRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

WorkerRegisterRequest = _reflection.GeneratedProtocolMessageType('WorkerRegisterRequest', (_message.Message,), {
  'DESCRIPTOR' : _WORKERREGISTERREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:WorkerRegisterRequest)
  })
_sym_db.RegisterMessage(WorkerRegisterRequest)

WorkerRegisterResponse = _reflection.GeneratedProtocolMessageType('WorkerRegisterResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKERREGISTERRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:WorkerRegisterResponse)
  })
_sym_db.RegisterMessage(WorkerRegisterResponse)

WorkerUpdateRequest = _reflection.GeneratedProtocolMessageType('WorkerUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _WORKERUPDATEREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:WorkerUpdateRequest)
  })
_sym_db.RegisterMessage(WorkerUpdateRequest)

WorkerUpdateResponse = _reflection.GeneratedProtocolMessageType('WorkerUpdateResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKERUPDATERESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:WorkerUpdateResponse)
  })
_sym_db.RegisterMessage(WorkerUpdateResponse)

WorkerDeregisterRequest = _reflection.GeneratedProtocolMessageType('WorkerDeregisterRequest', (_message.Message,), {
  'DESCRIPTOR' : _WORKERDEREGISTERREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:WorkerDeregisterRequest)
  })
_sym_db.RegisterMessage(WorkerDeregisterRequest)

WorkerDeregisterResponse = _reflection.GeneratedProtocolMessageType('WorkerDeregisterResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKERDEREGISTERRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:WorkerDeregisterResponse)
  })
_sym_db.RegisterMessage(WorkerDeregisterResponse)

LeaderRegisterRequest = _reflection.GeneratedProtocolMessageType('LeaderRegisterRequest', (_message.Message,), {
  'DESCRIPTOR' : _LEADERREGISTERREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:LeaderRegisterRequest)
  })
_sym_db.RegisterMessage(LeaderRegisterRequest)

LeaderRegisterResponse = _reflection.GeneratedProtocolMessageType('LeaderRegisterResponse', (_message.Message,), {
  'DESCRIPTOR' : _LEADERREGISTERRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:LeaderRegisterResponse)
  })
_sym_db.RegisterMessage(LeaderRegisterResponse)

PollLeaderRequest = _reflection.GeneratedProtocolMessageType('PollLeaderRequest', (_message.Message,), {
  'DESCRIPTOR' : _POLLLEADERREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:PollLeaderRequest)
  })
_sym_db.RegisterMessage(PollLeaderRequest)

PollLeaderResponse = _reflection.GeneratedProtocolMessageType('PollLeaderResponse', (_message.Message,), {
  'DESCRIPTOR' : _POLLLEADERRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:PollLeaderResponse)
  })
_sym_db.RegisterMessage(PollLeaderResponse)

PollLeaderJobRequest = _reflection.GeneratedProtocolMessageType('PollLeaderJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _POLLLEADERJOBREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:PollLeaderJobRequest)
  })
_sym_db.RegisterMessage(PollLeaderJobRequest)

PollLeaderJobResponse = _reflection.GeneratedProtocolMessageType('PollLeaderJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _POLLLEADERJOBRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:PollLeaderJobResponse)
  })
_sym_db.RegisterMessage(PollLeaderJobResponse)

PipelineStageRequest = _reflection.GeneratedProtocolMessageType('PipelineStageRequest', (_message.Message,), {
  'DESCRIPTOR' : _PIPELINESTAGEREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:PipelineStageRequest)
  })
_sym_db.RegisterMessage(PipelineStageRequest)

PipelineStageResponse = _reflection.GeneratedProtocolMessageType('PipelineStageResponse', (_message.Message,), {
  'DESCRIPTOR' : _PIPELINESTAGERESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:PipelineStageResponse)
  })
_sym_db.RegisterMessage(PipelineStageResponse)

NewJobLeaderRequest = _reflection.GeneratedProtocolMessageType('NewJobLeaderRequest', (_message.Message,), {
  'DESCRIPTOR' : _NEWJOBLEADERREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:NewJobLeaderRequest)
  })
_sym_db.RegisterMessage(NewJobLeaderRequest)

NewJobLeaderResponse = _reflection.GeneratedProtocolMessageType('NewJobLeaderResponse', (_message.Message,), {
  'DESCRIPTOR' : _NEWJOBLEADERRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:NewJobLeaderResponse)
  })
_sym_db.RegisterMessage(NewJobLeaderResponse)

LeaderHeartbeatRequest = _reflection.GeneratedProtocolMessageType('LeaderHeartbeatRequest', (_message.Message,), {
  'DESCRIPTOR' : _LEADERHEARTBEATREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:LeaderHeartbeatRequest)
  })
_sym_db.RegisterMessage(LeaderHeartbeatRequest)

LeaderHeartbeatResponse = _reflection.GeneratedProtocolMessageType('LeaderHeartbeatResponse', (_message.Message,), {
  'DESCRIPTOR' : _LEADERHEARTBEATRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:LeaderHeartbeatResponse)
  })
_sym_db.RegisterMessage(LeaderHeartbeatResponse)

LeaderJobRequest = _reflection.GeneratedProtocolMessageType('LeaderJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _LEADERJOBREQUEST,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:LeaderJobRequest)
  })
_sym_db.RegisterMessage(LeaderJobRequest)

LeaderJobResponse = _reflection.GeneratedProtocolMessageType('LeaderJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _LEADERJOBRESPONSE,
  '__module__' : 'leader_pb2'
  # @@protoc_insertion_point(class_scope:LeaderJobResponse)
  })
_sym_db.RegisterMessage(LeaderJobResponse)



_DISTRIBUTEDSERVER = _descriptor.ServiceDescriptor(
  name='DistributedServer',
  full_name='DistributedServer',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1907,
  serialized_end=2584,
  methods=[
  _descriptor.MethodDescriptor(
    name='AddClient',
    full_name='DistributedServer.AddClient',
    index=0,
    containing_service=None,
    input_type=_WORKERREGISTERREQUEST,
    output_type=_WORKERREGISTERRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUpdate',
    full_name='DistributedServer.GetUpdate',
    index=1,
    containing_service=None,
    input_type=_WORKERUPDATEREQUEST,
    output_type=_WORKERUPDATERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveClient',
    full_name='DistributedServer.RemoveClient',
    index=2,
    containing_service=None,
    input_type=_WORKERDEREGISTERREQUEST,
    output_type=_WORKERDEREGISTERRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RegisterLeader',
    full_name='DistributedServer.RegisterLeader',
    index=3,
    containing_service=None,
    input_type=_LEADERREGISTERREQUEST,
    output_type=_LEADERREGISTERRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PollLeader',
    full_name='DistributedServer.PollLeader',
    index=4,
    containing_service=None,
    input_type=_POLLLEADERREQUEST,
    output_type=_POLLLEADERRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='MarkStageCompleted',
    full_name='DistributedServer.MarkStageCompleted',
    index=5,
    containing_service=None,
    input_type=_PIPELINESTAGEREQUEST,
    output_type=_PIPELINESTAGERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PollLeaderJob',
    full_name='DistributedServer.PollLeaderJob',
    index=6,
    containing_service=None,
    input_type=_POLLLEADERJOBREQUEST,
    output_type=_POLLLEADERJOBRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateNewJob',
    full_name='DistributedServer.CreateNewJob',
    index=7,
    containing_service=None,
    input_type=_NEWJOBLEADERREQUEST,
    output_type=_NEWJOBLEADERRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CheckLeaderHeartbeat',
    full_name='DistributedServer.CheckLeaderHeartbeat',
    index=8,
    containing_service=None,
    input_type=_LEADERHEARTBEATREQUEST,
    output_type=_LEADERHEARTBEATRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ShareCurrentJob',
    full_name='DistributedServer.ShareCurrentJob',
    index=9,
    containing_service=None,
    input_type=_LEADERJOBREQUEST,
    output_type=_LEADERJOBRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DISTRIBUTEDSERVER)

DESCRIPTOR.services_by_name['DistributedServer'] = _DISTRIBUTEDSERVER

# @@protoc_insertion_point(module_scope)
