# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from .bootstrap_pb2 import (
    ServiceRegistrationStatusRequest,
    ServiceRegistrationStatusResponse,
    ServiceRegisterRequest
)


class BootstrapServerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RegisterService = channel.unary_unary(
                '/BootstrapServer/RegisterService',
                request_serializer=ServiceRegisterRequest.SerializeToString,
                response_deserializer=ServiceRegistrationStatusResponse.FromString,
                )
        self.GetServiceRegistrationStatus = channel.unary_unary(
                '/BootstrapServer/GetServiceRegistrationStatus',
                request_serializer=ServiceRegistrationStatusRequest.SerializeToString,
                response_deserializer=ServiceRegistrationStatusResponse.FromString,
                )


class BootstrapServerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def RegisterService(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetServiceRegistrationStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BootstrapServerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RegisterService': grpc.unary_unary_rpc_method_handler(
                    servicer.RegisterService,
                    request_deserializer=ServiceRegisterRequest.FromString,
                    response_serializer=ServiceRegistrationStatusResponse.SerializeToString,
            ),
            'GetServiceRegistrationStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetServiceRegistrationStatus,
                    request_deserializer=ServiceRegistrationStatusRequest.FromString,
                    response_serializer=ServiceRegistrationStatusResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'BootstrapServer', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class BootstrapServer(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def RegisterService(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/BootstrapServer/RegisterService',
            ServiceRegisterRequest.SerializeToString,
            ServiceRegistrationStatusResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetServiceRegistrationStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/BootstrapServer/GetServiceRegistrationStatus',
            ServiceRegistrationStatusRequest.SerializeToString,
            ServiceRegistrationStatusResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
